require 'xvfb'

describe Xvfb do
  before do
    ENV['DISPLAY'] = ":31337"
    stub_environment
  end

  describe 'launch options' do
    before do
      allow_any_instance_of(Xvfb).to receive(:ensure_xvfb_launched).and_return(true)
    end

    it "starts Xvfb" do
      expect(Process).to receive(:spawn).with(*(%w(/usr/bin/Xvfb :99 -screen 0 1280x1024x24 -ac)+[hash_including(:err)])).and_return(123)
      xvfb = Xvfb.new
    end

    it "allows setting screen dimensions" do
      expect(Process).to receive(:spawn).with(*(%w(/usr/bin/Xvfb :99 -screen 0 1024x768x16 -ac)+[hash_including(:err)])).and_return(123)
      xvfb = Xvfb.new(:dimensions => "1024x768x16")
    end
  end

  context 'with stubbed launch_xvfb' do
    before do
      allow_any_instance_of(Xvfb).to receive(:launch_xvfb).and_return(true)
    end

    context "instantiation" do
      context "when Xvfb is not installed" do
        before do
          allow(Xvfb::CliUtil).to receive(:application_exists?).and_return(false)
        end

        it "raises an error" do
          expect { Xvfb.new }.to raise_error(Xvfb::Exception)
        end
      end

      context "when Xvfb is already running and was started by this user" do
        before do
          allow(Xvfb::CliUtil).to receive(:read_pid).with('/tmp/.X99-lock').and_return(31337)
          allow(Xvfb::CliUtil).to receive(:process_running?).with(31337).and_return(true)
          allow(Xvfb::CliUtil).to receive(:process_mine?).with(31337).and_return(true)

          allow(Xvfb::CliUtil).to receive(:read_pid).with('/tmp/.X100-lock').and_return(nil)
        end

        context "and display reuse is allowed" do
          let(:options) { {:reuse => true} }

          it "should reuse the existing Xvfb" do
            expect(Xvfb.new(options).display).to eq 99
          end

          it "should not be destroyed at exit by default" do
            expect(Xvfb.new(options).destroy_at_exit?).to eq false
          end
        end

        context "and display reuse is not allowed" do
          let(:options) { {:reuse => false} }

          it "should pick the next available display number" do
            expect(Xvfb.new(options).display).to eq 100
          end

          context "and display number is explicitly set" do
            let(:options) { {:reuse => false, :display => 99} }

            it "should fail with an exception" do
              expect { Xvfb.new(options) }.to raise_error(Xvfb::Exception)
            end

            context "and autopicking is allowed" do
              let(:options) { {:reuse => false, :display => 99, :autopick => true} }

              it "should pick the next available display number" do
                expect(Xvfb.new(options).display).to eq 100
              end
            end
          end
        end
      end

      context 'when Xvfb is started, but by another user' do
        before do
          allow(Xvfb::CliUtil).to receive(:read_pid).with('/tmp/.X99-lock').and_return(31337)
          allow(Xvfb::CliUtil).to receive(:process_running?).with(31337).and_return(true)
          allow(Xvfb::CliUtil).to receive(:process_mine?).with(31337).and_return(false)

          allow(Xvfb::CliUtil).to receive(:read_pid).with('/tmp/.X100-lock').and_return(nil)
        end

        context "and display autopicking is not allowed" do
          let(:options) { {:autopick => false} }

          it "should reuse the display" do
            expect(Xvfb.new(options).display).to eq 99
          end
        end

        context "and display autopicking is allowed" do
          let(:options) { {:autopick => true} }

          it "should pick the next display number" do
            expect(Xvfb.new(options).display).to eq 100
          end
        end
      end
    end

    context "lifecycle" do
      let(:xvfb) { Xvfb.new }
      describe "#start" do
        it "switches to the xvfb server" do
          expect(ENV['DISPLAY']).to eq ":31337"
          xvfb.start
          expect(ENV['DISPLAY']).to eq ":99"
        end
      end

      describe "#stop" do
        it "switches back from the xvfb server" do
          expect(ENV['DISPLAY']).to eq ":31337"
          xvfb.start
          expect(ENV['DISPLAY']).to eq ":99"
          xvfb.stop
          expect(ENV['DISPLAY']).to eq ":31337"
        end
      end

      describe "#destroy" do
        before do
          allow(Xvfb::CliUtil).to receive(:read_pid).and_return(4444)
        end

        it "switches back from the xvfb server and terminates the xvfb session" do
          expect(Process).to receive(:kill).with('TERM', 4444)

          expect(ENV['DISPLAY']).to eq ":31337"
          xvfb.start
          expect(ENV['DISPLAY']).to eq ":99"
          xvfb.destroy
          expect(ENV['DISPLAY']).to eq ":31337"
        end
      end
    end

    context "#video" do
      let(:xvfb) { Xvfb.new }

      it "returns video recorder" do
        expect(xvfb.video).to be_a_kind_of(Xvfb::VideoRecorder)
      end

      it "returns the same instance" do
        recorder = xvfb.video
        expect(xvfb.video).to eq recorder
      end
    end
  end

private

  def stub_environment
    allow(Xvfb::CliUtil).to receive(:application_exists?).and_return(true)
    allow(Xvfb::CliUtil).to receive(:read_pid).and_return(nil)
    allow(Xvfb::CliUtil).to receive(:path_to).and_return("/usr/bin/Xvfb")
  end
end
